@@ -11,7 +11,7 @@ import win32event # We use events and the WaitFor[Single|Multiple]Objects functi
 import win32con   # constants.
 from serialutil import *
 
-VERSION = "$Revision: 1.31 $".split()[1]     #extract CVS version
+VERSION = "$Revision: 1.40 $".split()[1]     #extract CVS version
 
 #from winbase.h. these should realy be in win32con
 MS_CTS_ON  = 16
@@ -21,13 +21,7 @@ MS_RLSD_ON = 128
 
 def device(portnum):
     """Turn a port number into a device name"""
-    #the "//./COMx" format is required for devices >= 9
-    #not all versions of windows seem to support this propperly
-    #so that the first few ports are used with the DOS device name
-    if portnum < 9:
-        return 'COM%d' % (portnum+1) #numbers are transformed to a string
-    else:
-        return r'\\.\COM%d' % (portnum+1)
+    return 'COM%d' % (portnum+1) #numbers are transformed to a string
 
 class Serial(SerialBase):
     """Serial port implemenation for Win32. This implemenatation requires a 
@@ -43,16 +37,16 @@ class Serial(SerialBase):
             raise SerialException("Port must be configured before it can be used.")
         self.hComPort = None
         try:
-            self.hComPort = win32file.CreateFile(self.portstr,
+            self.hComPort = win32file.CreateFile(self.makeDeviceName(self.portstr),
                    win32con.GENERIC_READ | win32con.GENERIC_WRITE,
                    0, # exclusive access
                    None, # no security
                    win32con.OPEN_EXISTING,
-                   win32con.FILE_ATTRIBUTE_NORMAL | win32con.FILE_FLAG_OVERLAPPED,
+                   win32con.FILE_FLAG_OVERLAPPED,
                    None)
         except Exception, msg:
             self.hComPort = None    #'cause __del__ is called anyway
-            raise SerialException("could not open port: %s" % msg)
+            raise SerialException("could not open port %s: %s" % (self.portstr, msg))
         # Setup a 4k buffer
         win32file.SetupComm(self.hComPort, 4096, 4096)
 
@@ -60,7 +54,7 @@ class Serial(SerialBase):
         self._orgTimeouts = win32file.GetCommTimeouts(self.hComPort)
 
         self._rtsState = win32file.RTS_CONTROL_ENABLE
-        self._dtrState = win32file.RTS_CONTROL_ENABLE
+        self._dtrState = win32file.DTR_CONTROL_ENABLE
 
         self._reconfigurePort()
         
@@ -170,15 +164,26 @@ class Serial(SerialBase):
         """Close port"""
         if self._isOpen:
             if self.hComPort:
-                #Restore original timeout values:
-                win32file.SetCommTimeouts(self.hComPort, self._orgTimeouts)
-                #Close COM-Port:
+                try:
+                    # Restore original timeout values:
+                    win32file.SetCommTimeouts(self.hComPort, self._orgTimeouts)
+                except win32file.error:
+                    # ignore errors. can happen for unplugged USB serial devices
+                    pass
+                # Close COM-Port:
                 win32file.CloseHandle(self.hComPort)
+                win32file.CloseHandle(self._overlappedRead.hEvent)
+                win32file.CloseHandle(self._overlappedWrite.hEvent)
                 self.hComPort = None
             self._isOpen = False
 
     def makeDeviceName(self, port):
-        return device(port)
+        # the "\\.\COMx" format is required for devices other than COM1-COM8
+        # not all versions of windows seem to support this properly
+        # so that the first few ports are used with the DOS device name
+        if port.upper().startswith('COM') and int(port[3:]) <= 8:
+            return port
+        return '\\\\.\\' + port
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     
@@ -211,18 +216,20 @@ class Serial(SerialBase):
             read = ''
         return read
 
-    def write(self, s):
+    def write(self, data):
         """Output the given string over the serial port."""
         if not self.hComPort: raise portNotOpenError
+        if not isinstance(data, str):
+            raise TypeError('expected str, got %s' % type(data))
         #print repr(s),
-        if s:
+        if data:
             #~ win32event.ResetEvent(self._overlappedWrite.hEvent)
-            err, n = win32file.WriteFile(self.hComPort, s, self._overlappedWrite)
+            err, n = win32file.WriteFile(self.hComPort, data, self._overlappedWrite)
             if err: #will be ERROR_IO_PENDING:
                 # Wait for the write to complete.
                 #~ win32event.WaitForSingleObject(self._overlappedWrite.hEvent, win32event.INFINITE)
                 n = win32file.GetOverlappedResult(self.hComPort, self._overlappedWrite, 1)
-                if n != len(s):
+                if n != len(data):
                     raise writeTimeoutError
                 
 
@@ -237,16 +244,15 @@ class Serial(SerialBase):
         if not self.hComPort: raise portNotOpenError
         win32file.PurgeComm(self.hComPort, win32file.PURGE_TXCLEAR | win32file.PURGE_TXABORT)
 
-    def sendBreak(self):
+    def sendBreak(self, duration=0.25):
         """Send break condition."""
         if not self.hComPort: raise portNotOpenError
         import time
         win32file.SetCommBreak(self.hComPort)
-        #TODO: how to set the correct duration??
-        time.sleep(0.020)
+        time.sleep(duration)
         win32file.ClearCommBreak(self.hComPort)
 
-    def setRTS(self,level=1):
+    def setRTS(self, level=1):
         """Set terminal status line: Request To Send"""
         if not self.hComPort: raise portNotOpenError
         if level:
@@ -256,7 +262,7 @@ class Serial(SerialBase):
             self._rtsState = win32file.RTS_CONTROL_DISABLE
             win32file.EscapeCommFunction(self.hComPort, win32file.CLRRTS)
 
-    def setDTR(self,level=1):
+    def setDTR(self, level=1):
         """Set terminal status line: Data Terminal Ready"""
         if not self.hComPort: raise portNotOpenError
         if level:
@@ -298,17 +304,17 @@ class Serial(SerialBase):
 
 #Nur Testfunktion!!
 if __name__ == '__main__':
-    print __name__
+    s = Serial(0)
+    print s
+    
     s = Serial()
     print s
     
-    s = Serial(0)
-    print s
     
     s.baudrate = 19200
     s.databits = 7
     s.close()
-    s.port = 3
+    s.port = 0
     s.open()
     print s
 
